﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  [Item("Kotanchek function (2d)", "")]
  public class KotanchekFunction : RegressionBenchmark {

    private const string targetVariable = "Y";
    private static readonly List<string> inputVariables = new List<string>() { "X1", "X2" };
    private static readonly IntRange trainingPartition = new IntRange(0, 1000);
    private static readonly IntRange testPartition = new IntRange(1001, 2000);

    public override List<string> InputVariable {
      get { return inputVariables; }
    }

    public override string TargetVariable {
      get { return targetVariable; }
    }

    public override IntRange TrainingPartition {
      get { return trainingPartition; }
    }

    public override IntRange TestPartition {
      get { return testPartition; }
    }

    public KotanchekFunction() {
      Name = "Kotanchek function (2d)";
    }

    #region cloning
    protected KotanchekFunction(KotanchekFunction original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new KotanchekFunction(this, cloner);
    }
    #endregion

    protected override List<double> CalculateFunction(Dictionary<string, IList<double>> data) {
      double x1, x2;
      List<double> results = new List<double>();
      for (int i = 0; i < TestPartition.End; i++) {
        x1 = data[InputVariable.ElementAt(0)].ElementAt(i);
        x2 = data[InputVariable.ElementAt(1)].ElementAt(i);
        results.Add(Math.Exp(-Math.Pow(x1 - 1, 2)) / (Math.Pow(x2 - 2.5, 2) + 3.2));
      }
      return results;
    }

    protected override Dictionary<string, IList<double>> GenerateInput(Dictionary<string, IList<double>> data) {
      foreach (var variable in InputVariable) {
        data[variable] = RegressionBenchmark.generateUniformDistributedValues(TestPartition.End, new DoubleRange(0, 4));
      }
      data[TargetVariable] = CalculateFunction(data);
      return data;
    }
  }
}
